package it.cavallium.strangedb.java.objects.lists;

import it.cavallium.strangedb.functionalinterfaces.ConsumerWithIO;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.concurrent.CompletionException;
import java.util.concurrent.locks.ReentrantReadWriteLock;

public class ElementsArrayList<T> implements ElementsList<T> {

	ReentrantReadWriteLock readWriteLock = new ReentrantReadWriteLock(false);

	private final ArrayList<T> list;

	public ElementsArrayList() {
		this.list = new ArrayList<>();
	}

	public ElementsArrayList(Collection<T> collection) {
		this.list = new ArrayList<>(collection);
	}

	public ElementsArrayList(int initialCapacity) {
		this.list = new ArrayList<>(initialCapacity);
	}

	@Override
	public T get(int index) {
		return list.get(index);
	}

	@Override
	public void forEachParallelUnsorted(ConsumerWithIO<T> action) throws IOException {
		readWriteLock.readLock().lock();
		try {
			try {
				list.parallelStream().forEach((item) -> {
					try {
						action.accept(item);
					} catch (IOException ex) {
						throw new CompletionException(ex);
					}
				});
			} catch (CompletionException ex) {
				throw new IOException(ex.getCause());
			}
		} finally {
			readWriteLock.readLock().unlock();
		}
	}

	@Override
	public void add(T value) {
		readWriteLock.writeLock().lock();
		try {
			list.add(value);
		} finally {
			readWriteLock.writeLock().unlock();
		}
	}

	@Override
	public void update(int index, T value) {
		set(index, value);
	}

	@Override
	public void set(int index, T value) {
		readWriteLock.writeLock().lock();
		try {
			list.set(index, value);
		} finally {
			readWriteLock.writeLock().unlock();
		}
	}

	@Override
	public void add(int index, T value) {
		readWriteLock.writeLock().lock();
		try {
			list.add(index, value);
		} finally {
			readWriteLock.writeLock().unlock();
		}
	}

	@Override
	public T getLast() {
		readWriteLock.readLock().lock();
		try {
			return list.get(list.size() - 1);
		} finally {
			readWriteLock.readLock().unlock();
		}
	}

	@Override
	public boolean isEmpty() {
		readWriteLock.readLock().lock();
		try {
			return list.isEmpty();
		} finally {
			readWriteLock.readLock().unlock();
		}
	}

	@Override
	public int size() {
		readWriteLock.readLock().lock();
		try {
			return list.size();
		} finally {
			readWriteLock.readLock().unlock();
		}
	}

	public boolean contains(Object o) {
		readWriteLock.readLock().lock();
		try {
			return list.contains(o);
		} finally {
			readWriteLock.readLock().unlock();
		}
	}

	public ArrayList<T> asList() {
		readWriteLock.readLock().lock();
		try {
			return list;
		} finally {
			readWriteLock.readLock().unlock();
		}
	}
}
