package it.cavallium.strangedb.tests.query;

import it.cavallium.strangedb.java.annotations.DbDataType;
import it.cavallium.strangedb.java.annotations.DbField;
import it.cavallium.strangedb.java.annotations.DbPrimitiveField;
import it.cavallium.strangedb.java.annotations.DbPrimitiveType;
import it.cavallium.strangedb.java.database.IDatabaseTools;
import it.cavallium.strangedb.java.objects.EnhancedObject;

import java.io.IOException;

public class EMessage extends EnhancedObject {
	/**
	 * Message identifier, unique for the chat to which the message belongs.
	 */
	@DbPrimitiveField(id = 0, type = DbPrimitiveType.LONG, name = "id")
	public long id;
	/**
	 * Identifier of the user who sent the message; 0 if unknown. It is unknown for channel posts.
	 */
	@DbPrimitiveField(id = 1, type = DbPrimitiveType.INTEGER, name = "senderUserId")
	public int senderUserId;
	/**
	 * Chat identifier.
	 */
	@DbPrimitiveField(id = 2, type = DbPrimitiveType.LONG, name = "chatId")
	public long chatId;
	/**
	 * Information about the sending state of the message; may be null.
	 */
	@DbField(id = 0, type = DbDataType.OBJECT, name = "sendingState")
	public Object sendingState;
	/**
	 * True, if the message is outgoing.
	 */
	@DbPrimitiveField(id = 3, type = DbPrimitiveType.BOOLEAN, name = "isOutgoing")
	public boolean isOutgoing;
	/**
	 * True, if the message can be edited.
	 */
	@DbPrimitiveField(id = 4, type = DbPrimitiveType.BOOLEAN, name = "canBeEdited")
	public boolean canBeEdited;
	/**
	 * True, if the message can be forwarded.
	 */
	@DbPrimitiveField(id = 5, type = DbPrimitiveType.BOOLEAN, name = "canBeForwarded")
	public boolean canBeForwarded;
	/**
	 * True, if the message can be deleted only for the current user while other users will continue to see it.
	 */
	@DbPrimitiveField(id = 6, type = DbPrimitiveType.BOOLEAN, name = "canBeDeletedOnlyForSelf")
	public boolean canBeDeletedOnlyForSelf;
	/**
	 * True, if the message can be deleted for all users.
	 */
	@DbPrimitiveField(id = 7, type = DbPrimitiveType.BOOLEAN, name = "canBeDeletedForAllUsers")
	public boolean canBeDeletedForAllUsers;
	/**
	 * True, if the message is a channel post. All messages to channels are channel posts, all other messages are not channel posts.
	 */
	@DbPrimitiveField(id = 8, type = DbPrimitiveType.BOOLEAN, name = "isChannelPost")
	public boolean isChannelPost;
	/**
	 * True, if the message contains an unread mention for the current user.
	 */
	@DbPrimitiveField(id = 9, type = DbPrimitiveType.BOOLEAN, name = "containsUnreadMention")
	public boolean containsUnreadMention;
	/**
	 * Point in time (Unix timestamp) when the message was sent.
	 */
	@DbPrimitiveField(id = 10, type = DbPrimitiveType.INTEGER, name = "date")
	public int date;
	/**
	 * Point in time (Unix timestamp) when the message was last edited.
	 */
	@DbPrimitiveField(id = 11, type = DbPrimitiveType.INTEGER, name = "editDate")
	public int editDate;
	/**
	 * Information about the initial message sender; may be null.
	 */
	@DbField(id = 1, type = DbDataType.OBJECT, name = "forwardInfo")
	public Object forwardInfo;
	/**
	 * If non-zero, the identifier of the message this message is replying to; can be the identifier of a deleted message.
	 */
	@DbPrimitiveField(id = 12, type = DbPrimitiveType.LONG, name = "replyToMessageId")
	public long replyToMessageId;
	/**
	 * For self-destructing messages, the message's TTL (Time To Live), in seconds; 0 if none. TDLib will send updateDeleteMessages or updateMessageContent once the TTL expires.
	 */
	@DbPrimitiveField(id = 13, type = DbPrimitiveType.INTEGER, name = "ttl")
	public int ttl;
	/**
	 * Time left before the message expires, in seconds.
	 */
	@DbPrimitiveField(id = 14, type = DbPrimitiveType.DOUBLE, name = "ttlExpiresIn")
	public double ttlExpiresIn;
	/**
	 * If non-zero, the user identifier of the bot through which this message was sent.
	 */
	@DbPrimitiveField(id = 15, type = DbPrimitiveType.INTEGER, name = "viaBotUserId")
	public int viaBotUserId;
	/**
	 * For channel posts, optional author signature.
	 */
	@DbField(id = 2, type = DbDataType.OBJECT, name = "authorSignature")
	public String authorSignature;
	/**
	 * Number of times this message was viewed.
	 */
	@DbPrimitiveField(id = 16, type = DbPrimitiveType.INTEGER, name = "views")
	public int views;
	/**
	 * Unique identifier of an album this message belongs to. Only photos and videos can be grouped together in albums.
	 */
	@DbPrimitiveField(id = 17, type = DbPrimitiveType.LONG, name = "mediaAlbumId")
	public long mediaAlbumId;
	/**
	 * Content of the message.
	 */
	@DbField(id = 3, type = DbDataType.ENHANCED_OBJECT, name = "content")
	public EMessageContent content;
	/**
	 * Reply markup for the message; may be null.
	 */
	@DbField(id = 4, type = DbDataType.OBJECT, name = "replyMarkup")
	public Object replyMarkup;

	@Deprecated
	public EMessage() {

	}

	public EMessage(IDatabaseTools tools, EMessageContent content) throws IOException {
		super(tools);
		this.content = content;
	}
}
